package ru.dbotthepony.mc.otm.registry

import net.minecraft.data.BuiltinRegistries
import net.minecraft.tags.BlockTags
import net.minecraft.world.level.levelgen.VerticalAnchor
import net.minecraft.world.level.levelgen.feature.ConfiguredFeature
import net.minecraft.world.level.levelgen.feature.Feature
import net.minecraft.world.level.levelgen.feature.configurations.OreConfiguration
import net.minecraft.world.level.levelgen.placement.CountPlacement
import net.minecraft.world.level.levelgen.placement.HeightRangePlacement
import net.minecraft.world.level.levelgen.placement.InSquarePlacement
import net.minecraft.world.level.levelgen.placement.PlacedFeature
import net.minecraft.world.level.levelgen.structure.templatesystem.TagMatchTest
import net.minecraftforge.eventbus.api.IEventBus
import net.minecraftforge.registries.DeferredRegister
import net.minecraftforge.registries.RegistryObject
import ru.dbotthepony.mc.otm.OverdriveThatMatters

private val stone = TagMatchTest(BlockTags.STONE_ORE_REPLACEABLES)
private val deepslate = TagMatchTest(BlockTags.DEEPSLATE_ORE_REPLACEABLES)

private val target = listOf(
	OreConfiguration.target(stone, MBlocks.TRITANIUM_ORE.defaultBlockState()),
	OreConfiguration.target(deepslate, MBlocks.DEEPSLATE_TRITANIUM_ORE.defaultBlockState()),
)

object ДатьЛещаСломатьКолени {
	private val configuredFeatures = DeferredRegister.create(BuiltinRegistries.CONFIGURED_FEATURE.key(), OverdriveThatMatters.MOD_ID)
	private val placementConfigs = DeferredRegister.create(BuiltinRegistries.PLACED_FEATURE.key(), OverdriveThatMatters.MOD_ID)

	val tritaniumConfig: RegistryObject<ConfiguredFeature<*, *>> = configuredFeatures.register("tritanium_ore") { ConfiguredFeature(Feature.ORE, OreConfiguration(target, 9)) }

	val tritaniumPlacementNormal: RegistryObject<PlacedFeature> = placementConfigs.register("normal_tritanium") {
		PlacedFeature(
			tritaniumConfig.holder.get(),
			listOf(
				CountPlacement.of(8),
				InSquarePlacement.spread(),
				HeightRangePlacement.triangle(VerticalAnchor.absolute(0), VerticalAnchor.absolute(50))
			)
		)
	}

	val tritaniumPlacementDeepslate: RegistryObject<PlacedFeature> = placementConfigs.register("deep_tritanium") {
		PlacedFeature(
			tritaniumConfig.holder.get(),
			listOf(
				CountPlacement.of(10),
				InSquarePlacement.spread(),
				HeightRangePlacement.uniform(VerticalAnchor.aboveBottom(8), VerticalAnchor.absolute(0))
			)
		)
	}

	internal fun register(bus: IEventBus) {
		configuredFeatures.register(bus)
		placementConfigs.register(bus)
	}
}
